unit LobbyServer;

interface

uses Windows, SysUtils, ChatServer, Classes, IdTcpServer, ServerInfo, TestClient, StrUtils, Math, DateUtils,
  Constants;

var SeedId: longint;

{ TLobbyGame contiene le informazioni di una partita }
type
  TLobbyGame = class
  private
    AOwner: TComponent;
    m_Id: longint;
    m_Host: string;
    m_Port: integer;

  public
    constructor Create(AOwner: TComponent; Host: string; Port: integer);

    function IsAvaible: boolean;
    property Host: string read m_Host;
    property Port: integer read m_Port;
    property Id: longint read m_Id;
  end;

{ TLobbyServer eredita da TChatServer }
type
  TLobbyServer = class(TChatServer)
  private
    Games: TList;
    AOwner: TComponent;
    Motd: string;
    LastNicknameConnected: string;
    DateOfLastConnection: string;
  protected
    procedure ProcessCommand(StrData: string; AThread: TIdPeerThread); override;
    procedure DoLoginOperation(User: TChatUser); override;
    procedure DoDisconnectOperation(User: TChatUser); override;
    procedure SendMsgTo(User: TChatUser; Msg: string); override;
  public
    constructor Create(AOwner: TComponent; Port: integer); overload;
    constructor Create(AOwner: TComponent; Port: integer; Motd: string); overload;
    procedure SendUserListTo(AThread: TIdPeerThread);
    procedure SendUserListToAll;
    procedure SendGameListTo(AThread: TIdPeerThread);
    procedure SendGameListToAll;
    procedure AddGame(Game: TLobbyGame);
    function GetGame(Index: integer): TLobbyGame;
    function GetGameFromId(Id: integer): TLobbyGame;
    procedure DeleteGame(Index: integer);
    procedure DeleteGameWithId(Id: integer);
    function GetGamesCount: integer;
  end;

implementation

uses LobbyScanner;

{ Implementazione classe TLobbyGame }
constructor TLobbyGame.Create(AOwner: TComponent; Host: string; Port: integer);
begin
  self.AOwner := AOwner;
  self.m_Host := Host;
  self.m_Port := Port;


  { Assegna un valore id al game e incrementa il contatore }
  self.m_Id := SeedId;
  inc(SeedId);
end;

{ Funzione che controlla l'esistenza effettiva nella rete di una partita
chiamata bloccante }
function TLobbyGame.IsAvaible: boolean;
var
  ServerInfo: TServerInfo;
begin
  ServerInfo := TServerInfo.Create(AOwner,Host,Port);
  if ServerInfo.IsAvaible and (not ServerInfo.IsTheGameStarted) then Result := true
  else Result := false;
end;



{ Implementazione classe TLobbyServer }
constructor TLobbyServer.Create(AOwner: TComponent; Port: integer);
var
  LobbyScanner: TLobbyScanner;
begin
  { Inizializza la lista }
  Games := TList.Create;

  { Salva il riferimento ad AOwner }
  self.AOwner := AOwner;

  { Setta un messaggio di benvenuto predefinito... }
  self.Motd := 'Benvenuto nel server!';

  { Inizializza alcune vars... }
  LastNicknameConnected := '';
  DateOfLastConnection := '';
  
  inherited Create(AOwner, Port);

  { Inzializza il LobbyScanner }
  LobbyScanner := TLobbyScanner.Create(self);
end;

{ Versione overloadata che aggiunge il motd }
constructor TLobbyServer.Create(AOwner: TComponent; Port: integer; Motd: string);
begin
  Create(AOwner, Port);
  self.Motd := Motd;
end;

{ Aggiunge una partita alla lista }
procedure TLobbyServer.AddGame(Game: TLobbyGame);
begin
  Games.Add(Game);
end;

{ Rimuove una partita dalla lista }
procedure TLobbyServer.DeleteGame(Index: integer);
begin
  Games.Delete(Index);
end;

{ Rimuove una partita dalla lista partendo dal suo id }
procedure TLobbyServer.DeleteGameWithId(Id: integer);
var
  C: integer;
begin
  for C := 0 to Games.Count-1 do
    if GetGame(C).Id = Id then Games.Delete(C);
end;

{ Prende una partita partendo dal suo index }
function TLobbyServer.GetGame(Index: integer): TLobbyGame;
begin
  Result := TLobbyGame(Games.Items[Index]);
end;

{ Prende una partita partendo dal suo id }
function TLobbyServer.GetGameFromId(Id: integer): TLobbyGame;
var
  C: integer;
  Game: TLobbyGame;
begin
  for C := 0 to Games.Count-1 do
  begin
    Game := GetGame(C);
    if Game.Id = Id then begin
      Result := Game;
      exit;
    end;
  end;
  raise Exception.Create('Si  cercato di prendere l''oggetto TLobbyGame con Id: ' + IntToStr(Id) + ', ma non  stato trovato nell''elenco. Contattare il produttore.');
end;

{ Ridefiniamo la procedura process command per aggiungere nuovi comandi... }
procedure TLobbyServer.ProcessCommand(StrData: string; AThread: TIdPeerThread);
var
  Command: string;
  TestClient: TTestClient;
  NewNickname: string;
  OldNickname: string;
  User: TChatUser;
begin
  { Ok, aggiungiamo quel che ci serve, il resto lo ereditiamo dalla chat base }
  Command := GetCommand(StrData);

  { Parsa questi pacchetti solamente se sono di un utente loggato }
  if IsThisPeerThreadInList(AThread) then begin

    { Un utente ha richiesto la lista... }
    if Command = '/GIMMETHEUSERLIST' then begin
      SendUserListTo(AThread);
    end

    { Elaboriamo i comandi digitati dalla chat... irc mode }
    else if Command = '/MSG' then begin
      { Un giocatore ha digitato il comando /NICK }
      if AnsiUpperCase(AnsiLeftStr(GetToken(StrData,1),Length('/NICK '))) = '/NICK ' then begin
        User := GetUser(AThread);
        OldNickname := User.Nickname;
        NewNickname := AnsiRightStr(GetToken(StrData,1),Length(GetToken(StrData,1)) - Length('/NICK '));

        { Se qualcuno cerca di farmi un buffer overflow... }
        if Length(NewNickname) > 20 then NewNickname := AnsiLeftStr(NewNickname,20);

        { Se il nickname esiste gia, aggiungi un numero casuale alla fine... }
        if NicknameAlreadyExists(NewNickname) then NewNickname := NewNickname + IntToStr(RandomRange(0,9999));
        User.Nickname := NewNickname;

        SendTo(AThread,'/NICKNAMECHANGEDSUCCESSFUL'+SEPARATOR+NewNickname);
        SendMsgToAll(MakeServerMessage(OldNickname + '  ora conosciuto come ' + NewNickname));
        SendUserListToAll;

        { Previene che questo messaggio venga elaborato dalla classe base }
        exit;
      end;
    end

    { Un giocatore vuole avviare una partita
    Command = '/ADDTHISGAME_Ip/Host_Port'
    NOTA BENE: OBSOLETO/DEPRECATO }
    else if Command = '/ADDTHISGAME' then begin
      AddGame(TLobbyGame.Create(AOwner,GetToken(StrData,1),StrToInt(GetToken(StrData,2))));

      { Comunica ai clients la nuova lista... }
      SendGameListToAll;
    end

    { Nuova versione
    Command = '/ISTARTANEWGAME_Port' }
    else if Command = '/ISTARTANEWGAME' then begin
      AddGame(TLobbyGame.Create(AOwner,GetUser(AThread).Ip,StrToInt(GetToken(StrData,1))));

      { Comunica ai clients la nuova lista... }
      SendGameListToAll;
    end

    else if Command = '/GIMMETHEGAMELIST' then begin
      SendGameListTo(AThread);
    end

    { C' un client che ha bisogno del test di connessione...
    Command = '/IMREADYFORTHECONNECTIONTEST_PortToTest' }
    else if Command = '/IMREADYFORTHECONNECTIONTEST' then begin
      { l'oggetto TestClient opera in un thread secondario e notifica in maniera autonoma il risultato
      del test }
      TestClient := TTestClient.Create(AOwner,GetUser(AThread).Ip,StrToInt(GetToken(StrData,1)),AThread);
    end;
  end
  else begin
    { Altrimenti parsa pure questi (se  di un utente non loggato) }

    { La versione di questo client combacia con la nostra? }
    if Command = '/MYLOBBYVERSIONIS' then begin
      if GetToken(StrData,1) = LOBBYVERSION then SendTo(AThread,'/URLOBBYVERSIONISSUPPORTED')
      else SendTo(AThread,'/URLOBBYVERSIONISNOTSUPPORTED'+SEPARATOR+LOBBYVERSION);
    end;
  end;

  inherited ProcessCommand(StrData,AThread);
end;

{ Ridefinisce la procedura per inviare i messaggi agli utenti }
procedure TLobbyServer.SendMsgTo(User: TChatUser; Msg: string);
begin
  { Se il messaggio  di un utente (comincia con '<') aggiungi la data, altrimenti niente }
  if AnsiStartsStr('<',Msg) then inherited SendMsgTo(User,'['+TimeToStr(Time)+'] ' + Msg)
  else inherited SendMsgTo(User,Msg);
end;


{ Procedura per inviare la lista degli utenti connessi ad un utente }
procedure TLobbyServer.SendUserListTo(AThread: TIdPeerThread);
var
  UserListStr: string;
  C: integer;
begin
  UserListStr := '/USERLIST' + SEPARATOR + IntToStr(Clients.Count);

  { Creiamo la lista in un formato stringa }
  for C := 0 to Clients.Count-1 do
    UserListStr := UserListStr + SEPARATOR + GetUser(C).Nickname;

  { E inviamola all'utente }
  SendTo(AThread,UserListStr);
end;

{ Procedura per inviare a tutti i clients connessi la lista degli utenti }
procedure TLobbyServer.SendUserListToAll;
var
  C: integer;
begin
  for C := 0 to Clients.Count-1 do
    SendUserListTo(GetUser(C).AThread);
end;

{ Procedura per inviare la lista della partite a tutti gli utenti }
procedure TLobbyServer.SendGameListToAll;
var
  C: integer;
begin
  for C := 0 to Clients.Count-1 do
    SendGameListTo(GetUser(C).AThread);
end;

{ Funzione per inviare la lista delle partite ad un utente }
procedure TLobbyServer.SendGameListTo(AThread: TIdPeerThread);
var
  GameListStr: string;
  Game: TLobbyGame;
  C: integer;
begin
  GameListStr := '/GAMELIST' + SEPARATOR + IntToStr(Games.Count);

  { Crea la lista in formato stringa }
  for C := 0 to Games.Count-1 do
  begin
    Game := GetGame(C);
    GameListStr := GameListStr + SEPARATOR + Game.Host + SEPARATOR + IntToStr(Game.Port);
  end;

  { E la invia }
  SendTo(AThread,GameListStr);
end;

{ Funzione che ritorna il numero di partite presenti nella lista  }
function TLobbyServer.GetGamesCount: integer;
begin
  Result := Games.Count;
end;

{ Procedura richiamata subito dopo il login }
procedure TLobbyServer.DoLoginOperation(User: TChatUser);
begin
  { Invia la lista degli utenti, quella delle partite,
  invia il messaggio di benvenuto al client e notifica l'entrata nel lobby
  a tutti quanti, mostra l'ultimo utente connesso e quando }
  
  SendUserListToAll;
  SendGameListTo(User.AThread);

  SendMsgTo(User,''); //Riga vuota
  SendMsgTo(User,MakeServerMessage('Questo server lobby  aggiornato alla versione ' + LOBBYVERSION + ', vi prego di segnalare qualsiasi malfunzionamento del gioco all''indirizzo ' + SUBMITBUGPAGE));
  SendMsgTo(User,'Digita /NICK <nuovo nickname> per cambiare il tuo nickname');
  SendMsgTo(User,''); //Riga vuota
  SendMsgTo(User,MakeServerMessage(Motd));


  { Se qualcuno si  gi connesso, vuol dire che abbiamo qualcosa da stampare }
  if LastNicknameConnected <> '' then SendMsgTo(User,MakeServerMessage('L''ultimo giocatore che si connesso al lobby si chiamava '+LastNicknameConnected+' ed  entrato alle ' + DateOfLastConnection));

  { Salva il nuovo nickname dell'ultimo utente, e la data di connessione }
  LastNicknameConnected := User.Nickname;
  DateOfLastConnection := TimeToStr(Time);


  SendMsgToAll(MakeServerMessage(User.Nickname + '  entrato nel lobby'));
end;

{ Un client se n' andato.. }
procedure TLobbyServer.DoDisconnectOperation(User: TChatUser);
begin
  SendMsgToAll(MakeServerMessage(User.Nickname + '  uscito dal lobby'));
  RemoveUser(User.AThread);
  SendUserListToAll;
end;

initialization

SeedId := 0;

end.
